'' =================================================================================================
''
''   File....... jm_ds18b20_demo.spin
''   Purpose....
''   Author..... Jon "JonnyMac" McPhalen
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com
''   Started....
''   Updated.... 29 JAN 2019
''
'' =================================================================================================


con { timing }

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000                                          ' use 5MHz crystal

  CLK_FREQ = (_clkmode >> 6) * _xinfreq                         ' system freq as a constant
  MS_001   = CLK_FREQ / 1_000                                   ' ticks in 1ms
  US_001   = CLK_FREQ / 1_000_000                               ' ticks in 1us
                                                                 
                                                                 
con { io pins }                                                  
                                                                   
  RX1  = 31                                                     ' programming / terminal
  TX1  = 30                                                      
                                                                 
  SDA1 = 29                                                     ' eeprom / i2c
  SCL1 = 28                                                      
                                                                 
  OWX  =  0
                                                             
                                                    
obj                                                              
                                                                 
' main                                                          ' * master Spin cog
  time : "jm_time_80"                                           '   timing and delays
  io   : "jm_io_basic"                                          '   essential io
  ow   : "jm_1-wire"                                            ' * 1-Wire driver
  term : "jm_fullduplexserial"                                  ' * serial IO for terminal
                                                                 
' * uses cog when loaded                                         
                                                                 
                                                                 
var                                                              
                                                                 
                                                                 
pub main | status, t0, idx, tmpC, tmpF                                                          
                                                                 
  setup

  term.rxflush                                                  ' open and enable terminal first
  term.rx
  term.tx(term#CLS)
  
  status := ow.reset                                            ' check for device
  
  if (status == %10)                                            ' good 1W reset
    read_sn(@SN0)                                               ' read the serial #
    term.tx(term#CLS)
    repeat idx from 0 to 7                                      ' display serial # bytes
      term.hex(SN0[idx], 2)
      term.tx(" ")

    t0 := cnt                                                   ' sync timing loop
    repeat
'     tmpC := read_tc                                           ' read the temperature
      tmpC := read_tca(@SN0)                                    ' read from specific address

      string_xy(string("tmpC = "), 0, 2)                        ' show tC
      term.rjdec(tmpC / 10000, 3, " ")                          ' whole degrees
      term.tx(".")
      term.rjdec(tmpC // 10000, 4, "0")                         ' fractional degrees

      tmpF := tmpC * 9 / 5 + 32_0000                            ' convert to Fahrenheit

      string_xy(string("tmpF = "), 0, 3)                        ' show tF
      term.rjdec(tmpF / 10000, 3, " ")                          ' whole degrees
      term.tx(".")
      term.rjdec(tmpF // 10000, 4, "0")                         ' fractional degrees

      waitcnt(t0 += CLK_FREQ)  
      
  else
    term.tx(term#CLS)
    case status
      %00 : term.str(string("Bus Short"))
      %01 : term.str(string("Bus Error"))
      %11 : term.str(string("No Device"))  

    term.txflush                                                ' let message finish
    io.high(TX1)                                                ' force TX to idle
    term.stop                                                   ' unload terminal cog

    ow.stop                                                     ' unload 1W cog
       
    repeat
      waitcnt(0)                                                ' low-power until reset
    

pub string_xy(p_str, x, y)

  term.tx(term#CRSR_XY)
  term.tx(x)
  term.tx(y)
  term.str(p_str)
    

pub setup                                                        
                                                                 
'' Setup IO and objects for application                          
                                                                 
  time.start                                                    ' setup timing & delays
                                                                 
  io.start(0, 0)                                                ' clear all pins (master cog)
  
  ow.start(OWX)                                                 ' start 1-Wire  
                                                           
  term.start(RX1, TX1, %0000, 115_200)                          ' start serial for terminal
                                                                 

con

  { ------------- }
  {  1 - W I R E  }
  { ------------- }


con

  RD_ROM     = $33                                              ' 1W ROM commands
  MATCH_ROM  = $55
  SKIP_ROM   = $CC
  SRCH_ROM   = $F0
  ALARM_SRCH = $EC
  
  CVRT_TEMP  = $44                                              ' DS18xxx commands
  WR_SPAD    = $4E
  RD_SPAD    = $BE
  COPY_SPAD  = $48 
  RD_EE      = $B8
  RD_POWER   = $B4 


dat

  SN0           byte    $00, $00, $00, $00, $00, $00, $00, $00


pub read_sn(p_sn)

'' Reads serial number from 1W device
'' -- stores in array at p_sn
'' -- only connect one device at a time for this method

  ow.reset
  ow.write(RD_ROM)
  repeat 8
    byte[p_sn++] := ow.read


pub read_tc | tmpC

'' Reads temperature from DS18b20
'' -- skips id, so only one device can be connected
'' -- returns degrees C in 0.0001 degree units

  ow.reset               
  ow.write(SKIP_ROM)                                            ' no serial number needed         
  ow.write(CVRT_TEMP)                                           ' start temp conversion
  repeat                                                        ' let conversion finish                 
    tmpC := ow.rdbit     
  until (tmpC == 1)    
  ow.reset               
  ow.write(SKIP_ROM)          
  ow.write(RD_SPAD)                                             ' read scratchpad         
  tmpC := ow.read                                               ' lsb of temp      
  tmpC |= ow.read << 8                                          ' msb of temp

  tmpC := ~~tmpC * 625                                          ' extend sign, 0.0001 units

  return tmpC


pub read_tca(p_sn) | tmpC, idx

'' Reads temperature from DS18b20 at specific address
'' -- returns degrees C in 0.0001 degree units
'' -- p_sn is pointer to byte array with serial #

  ow.reset               
  ow.write(MATCH_ROM)                                           ' use serial #
  repeat idx from 0 to 7                                        ' send serial #
    ow.write(byte[p_sn][idx])          
  ow.write(CVRT_TEMP)                                           ' start conversion          
  repeat                                                        ' let conversion finish                 
    tmpC := ow.rdbit     
  until (tmpC == 1)    
  ow.reset               
  ow.write(MATCH_ROM)
  repeat idx from 0 to 7
    ow.write(byte[p_sn][idx])            
  ow.write(RD_SPAD)                                             ' read scratchpad     
  tmpC := ow.read                                               ' lsb of temp      
  tmpC |= ow.read << 8                                          ' msb of temp

  tmpC := ~~tmpC * 625                                          ' extend sign, 0.0001 units

  return tmpC


                                   
dat { license }

{{

  Copyright (C) 2014-2019 Jon McPhalen
  -- All Rights Reserved
  -- Attribution Required for derivative works: "Based on code by Jon McPhalen"

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}